      SUBROUTINE GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )

C            VERSION 3             24 JULY 2000
C
C    Address questions and comments to
C         Gordon D. Holman
C         NASA/Goddard Space Flight Center
C         Code 682
C         Greenbelt, MD 20771
C         Office phone: (301) 286-4636
C		E-Mail: holman@stars.gsfc.nasa.gov
C
C    Purpose -
C         The program computes values of the emissivity and absorption 
C         coefficients, j and K, resp., from the gyrosynchrotron 
C         radiation formulas.  This program uses three subprograms,
C		based upon subprograms found in "Numerical recipies in
C		FORTRAN 77".  The first and second are DMLIN and DCADRE.  
C		These two evaluate the integrals in the expressions for j
C         and K.  The third routine is MMBSJN, which calculates the
C		values of Bessel functions at a given point.  DMLIN uses a
C	    Gaussian Quadrature method, and will use up to 256 Gauss
C	    points to evaluate the integral.  DCADRE employs the method
C         of Romberg extrapolation, and subdivides the interval of
C         integration until the error tolerance is satisfied. MMBSJN
C	    determines the Bessel functions via Steed's method. This 
C		program reports all errors, from any section of the code, to
C	    the user.  However, fatal errors cause the course of the
C	    calculations to abort immediately after the error has been 
C		reported to the user.
C                                                                        
C    Files used -  
C		DCADRE.FOR			Romberg integration routine
C		DMLIN.FOR			Gaussian Quadrature integration routine
C		FDBPL.FOR			Electron distribution function and its derivatives
C		GETGSE.FOR			Core Routines - Compute j & K - this file
C		GETGSE.INC			Include file - common blocks, parameter definitions
C         GYROSPECTRUM.FOR	Main Program - Calls Subroutine GETGSE
C		MMBSJN.FOR			Computes values of Bessel functions
C
C    Terminal I/O - 		                                             
C         IUNITA     Defines device unit number for reading information  
C                    interactively from the user terminal keyboard.      
C                    Currently assigned to unit number 5.                 
C         IUNITB     Defines device unit number for writing information  
C                    to user's terminal screen.  Currently assigned to   
C                    unit number 6.                                      
C                                                                        
C    Subroutines -                                                       
C         DISTRN     Electron distribution function and its first derivatives                          
C         GETGSE     Governs computation of gyrosynchrotron emission  
C
C    Documentation - 
C         GyroCode.rtf  
  
      IMPLICIT NONE
  
C     Purpose -   This routine is in charge of making the calls to the 
C        subroutines to 1) initialize those variables whose values do 
C        not change in the evaluation of the integrals, and 2) to 
C        evaluate the integrals.
  
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    IERGQ, IERROM
  
      REAL*8       VALUEJ, VALUEK

C                  Initialize several variables common throughout the 
C                  program.
         CALL INITLZ ( )
         IF( (IERATH .NE. 0) .OR. (IERIND .NE. 0) .OR. 
     1       (IERNS  .NE. 0) .OR. (IERGAM .NE. 0) ) GOTO 99
	  
C                  Get the value of the integrals for J (emissivity) 
C                  and K (absorption coefficient).  This subroutine 
C                  uses the subprogram DMLIN, and a user specified 
C                  function for the distribution function.
         CALL GETJK  ( VALUEJ, VALUEK, IERGQ, IERROM )
  
  
 99   RETURN
      END

      SUBROUTINE INITLZ ( )
  
C    Purpose -     Initialize variables used throughout the program, 
C       and determine the range for the summing in S.  These values 
C       will be used unchanged throughout the program.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       RNUX
  
  
            IF( MODE .EQ. 1 )  THEN
C                  Test wave frequency for ORDINARY MODE.  If wave 
C                  frequency is too low, tell user to give input again.
                  IF( RNU .GT. RNUP )  THEN
                     CONTINUE
                  ELSE
                     IERNU  =  1
                     GOTO 99
                  ENDIF
            ELSE
C                  Test wave frequency for EXTRAORDINARY MODE.  If wave 
C                  frequency is too low, tell user to give input again.
               RNUX = DSQRT( (RNUP**2) + ((RNUB/2.0D+00)**2) )
               RNUX = RNUX + (RNUB/2.0D+00)
                  IF( RNU .GT. RNUX )  THEN
                     CONTINUE
                  ELSE
                     IERNU  = -1
                     GOTO 99
                  ENDIF
            ENDIF
  
  
C                  Determine THETA in radians, then get sin and cosine 
C                  of THETA.  
         IF( THETAD .EQ. 90.0D+00 )  THEN
            THETAR = PI/(2.0D+00)
            CTHETA = 0.00D+00
            STHETA = 1.00D+00
         ELSE IF( THETAD .EQ. 180.0D+00 ) THEN
	      THETAR = PI
	      CTHETA = -1.00D+00
	      STHETA =  0.00D+00
         ELSE
            THETAR = DEGRAD * THETAD
            CTHETA = DCOS( THETAR )
            STHETA = DSIN( THETAR )
         ENDIF
  
  
C                  Get value of INDEX OF REFRACTION.
         CALL GETIND ( )
  
  
C                  Check error return code from above and the get 
C                  limits of summing the integrations.
         IF( IERIND .EQ. 0 )  THEN  
               IF( CTHETA .EQ. 0.00D+00 )THEN
C                  To get limits at THETA = 90 degrees.
                  CALL SLIM90 ( )
               ELSE
C                  Get value of A_THETA (when THETA not equal 90 deg.)
                  CALL GETATH ( )
C                  To get limits at THETA not equal 90 degrees.
                  CALL SLIMIT ( )
               ENDIF
         ELSE
               IERNS = -3
         ENDIF
  
  
 99   RETURN
      END

      SUBROUTINE GETATH ( )
  
C    Purpose -     Get the value for A-THETA, as a function of the 
C       wave frequency and plasma and cyclotron frequencies, and the 
C       wave normal angle and the mode of wave polarization.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       ATERM, BTERM, DENOM, DETER
  
  
C                  Determine value of two terms, "A" & "B".  The 
C                  expressions of A-THETA for the ORDINARY MODE(=+1) 
C                  and the EXTRAORDINARY MODE (=-1) depends on these 
C                  two terms only.
            ATERM = - ((RNU/RNUB) * STHETA)**2
            BTERM = (2.D+00 * (RNU/RNUB) * CTHETA)
            BTERM = BTERM * ( ((RNUP/RNUB)**2) - ((RNU/RNUB)**2) )
  
  
C                  Evaluate term in denominator under square root sign.
            DETER = DSQRT( ((ATERM**2) + (BTERM**2)) )
  
  
               IF( MODE .EQ. 1 )  THEN
C                  Evaluate denominator for ORDINARY MODE (=+1).  
                  DENOM =  ATERM  +  DETER
               ELSE
C                  Evaluate denominator for EXTRAORDINARY MODE (=-1).
                  DENOM =  ATERM  -  DETER
               ENDIF
  
  
C                  If the denominator is zero, then give the error 
C                  return code variable a non-zero value (to show an 
C                  error has occurred) and set the return A-THETA 
C                  value to the flag value to indicate a problem has 
C                  occurred in the calculations (in this case, a 
C                  singularity).  
               IF( DENOM .EQ. 0.00D+00 )  THEN
                  IERATH = -1
                  ATHET  = DFLAG
               ELSE
                  ATHET  = - BTERM / DENOM
               ENDIF
  
  
C                 Computations completed for A-THETA.  Return to 
C                 calling unit.
      RETURN
      END

      SUBROUTINE GETIND ( )
  
C    Purpose -     Get the value for INDEX of refraction, as a function 
C       of the wave frequency and plasma and cyclotron frequencies, and 
C       the wave normal angle and the mode of wave polarization.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       DETER, DENOM, ATERM, BTERM, CTERM
  
  
C                  Determine three terms, "A" & "B" & "C".
            ATERM = ((RNU/RNUB)**2) * (STHETA**2)
            BTERM = 2.D+00 * ( ((RNUP/RNUB)**2) - ((RNU/RNUB)**2) )
            CTERM = BTERM * (RNU/RNUB) * CTHETA
  
  
C                  Get the term in the square root in the denominator.
            DETER = (ATERM**2) + (CTERM**2)
  
  
C                  Get the value of the denominator for the square of 
C                  the index of refraction, RINDX.  NOTE WELL that the 
C                  only difference between the two polarization modes 
C                  in this subroutine is in the PLUS or MINUS sign 
C                  before the square root is taken of the value DETER.
               IF( MODE .EQ. 1 )  THEN
C                  Evaluate the expression in the denominator for the 
C                  ORDINARY MODE (=+1).  
                  DENOM = (-ATERM) - (((RNU/RNUB)**2) * BTERM) + 
     1             (DSQRT( DETER ))
               ELSE
C                  Evaluate the expression in the denominator for the 
C                  EXTRAORDINARY MODE (=-1).  
                  DENOM = (-ATERM) - (((RNU/RNUB)**2) * BTERM) - 
     1             (DSQRT( DETER ))
               ENDIF
  
  
C                  If the denominator is zero, then give the flag value 
C                  to the INDEX OF REFRACTION, RINDX.  Otherwise, 
C                  evaluate the square of the RINDX.  Then get square 
C                  root if the value is not less than zero.
               IF( DENOM .EQ. 0.00D+00 )  THEN
                        IERIND = -1
                        RINDX  = DFLAG
               ELSE
                  RINDX   =  1.0D+00 
     1             + ( (BTERM * ((RNUP/RNUB)**2)) / DENOM )
                     IF( RINDX .LT. 0.00D+00 )  THEN
                        IERIND = -2
                        RINDX  = DFLAG
                     ELSE
                        IERIND =  0
                        RINDX  = DSQRT( RINDX )
                     ENDIF
               ENDIF
  
  
C                  Computations completed for the REFRACTIVE 
C                  INDEX.  Return to calling unit.
      RETURN
      END

      SUBROUTINE SLIM90 ( )
  
C    Purpose -     Get the limits of the summing index, NS (or "S"), 
C       from the input values of RNU, RNUB, GAMMIN & GAMMAX.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       SMIN, SMAX
  
  
C                  Get value of limits to index range in real numbers.
            SMIN  = GAMMIN * RNU / RNUB
            SMAX  = GAMMAX * RNU / RNUB
  
  
C                  Test for the appropriate value of the index limit 
C                  for NSMIN.  If SMIN equals the truncated integer 
C                  value, then NSMIN = SMIN.  Otherwise, truncate SMIN 
C                  and add 1.
            IF( (DINT(SMIN)) .EQ. SMIN )  THEN
               NSMIN  = IDINT( SMIN )
            ELSE
               NSMIN  = IDINT( SMIN )  +  1
            ENDIF
  
C                  Obtain the appropriate value of the index limit 
C                  for NSMAX.
            NSMAX  = IDINT( SMAX )
  
      RETURN
      END

      SUBROUTINE SLIMIT ( )
  
C    Purpose -     Get the limits for the summing of the integrations.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NSFIX
  
      REAL*8       GAMMAI, GAMMAF, ATERM
  
  
C                  Calculate value under square root sign, ATERM, and 
C                  check that it is not negative.  If so, set error 
C                  indicator and return to calling unit.  Otherwise, 
C                  determine square root, and calculate minimum value 
C                  allowed for NS, NSFIX.
            ATERM = (1.0D+00 - ((RINDX*CTHETA)**2))
               IF( ATERM .LT. 0.00D+00 )  THEN
                  IERNS = -1
                  GOTO 99
               ELSE
                  ATERM = (RNU/RNUB) * (DSQRT( ATERM ))
                  NSFIX = 1  +  (IDINT ( ATERM ))
               ENDIF
  
  
C                  Check the LOWER LIMIT to the summing of NS.  Get 
C                  limits of integration in GAMMA by calling GAMLIM, 
C                  for the current value of NS.
            DO  NS = NSFIX, NDIMEN
               CALL GAMLIM ( GAMMAI, GAMMAF )
CCC      write(16,*) ' NS, GAMMIN, GAMMAF=', NS, SNGL(GAMMIN), 
CCC     1    SNGL(GAMMAF)
                  IF( IERGAM .NE. 0 )  GOTO 99
               NSMIN  =  NS
C                  Test lower limit of summing index.
                  IF( (GAMMAF .LT. GAMMIN) )  THEN
                     CONTINUE
                  ELSE
                     GOTO 10
                  ENDIF
            END DO
  
C                  Check the UPPER LIMIT to the summing of NS.  Get 
C                  limits of integration in GAMMA by calling GAMLIM, 
C                  for the current value of NS.
 10         DO  NS = NSFIX, NDIMEN
               CALL GAMLIM ( GAMMAI, GAMMAF )
CCC      write(16,*) ' NS, GAMMAX, GAMMAI=', NS, SNGL(GAMMAX), 
CCC     1    SNGL(GAMMAI)
                  IF( IERGAM .NE. 0 )  GOTO 99
               NSMAX  =  NS - 1
C                  Test upper limit of summing index.
                  IF( GAMMAX .LT. GAMMAI )  THEN
                     GOTO 12
                  ELSE
                     CONTINUE
                  ENDIF
            END DO
  
  
C                  If the values of NSMIN or NSMAX is greater than 
C                  (NDIMEN-2), then the calculation of the emissivity, 
C                  J, and the absorption coefficient, K, will not 
C                  include all the terms needed in the sum over the 
C                  index NS.  Set error code if this is true.
 12         IF( (NDIMEN-2) .LT. NSMAX )  IERNS = -3
            IF( (NDIMEN-2) .LT. NSMIN )  IERNS = -2
CCC      write(16,*) ' NSFIX, NSMIN, NSMAX=', SNGL(NSFIX), SNGL(NSMIN), 
CCC     1 SNGL(NSMAX)
  
 99   RETURN
      END

      SUBROUTINE GAMLIM ( GAMMAI, GAMMAF )
  
C    Purpose -     Get the value for the limits of integration in GAMMA,
C       as a function S, of the wave frequency and plasma and cyclotron 
C       frequencies, and the wave normal angle and the mode of wave 
C       polarization.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       GAMMAI, GAMMAF, X, Y, Z, DENOM, DETON
  
  
C                  Calculate the three terms used in the expression.
            X = (DFLOAT(NS)) * (RNUB / RNU)
            Y = RINDX * (DABS(CTHETA))
            Z = (X**2)  +  (Y**2)
  
C                  Then check whether the term under the square root is 
C                  less than zero.  This is part of the denominator 
C                  of GAMMAI & GAMMAF.
               IF( (Z-1.0D+00) .LT. 0.00D+00 )  THEN
                  IERGAM = -1
                  GAMMAI = DFLAG
                  GAMMAF = DFLAG
               ELSE
                  DETON = Y * DSQRT( (Z - 1.0D+00) )
                  DENOM = X + DETON
C                  Get value of the denominator for GAMMAI.  Then test 
C                  whether denominator is zero.  
                     IF( DENOM .EQ. 0.00D+00 )  THEN
                        IERGAM = -2
                        GAMMAI = DFLAG
                        GOTO 99
                     ELSE
                        IERGAM =  0
                        GAMMAI = Z / DENOM
                     ENDIF
C                  Get value for the denominator of GAMMAF.  Then test 
C                  whether denominator is zero or if GAMMAI had error.
                  DENOM = X - DETON
                     IF( DENOM .EQ. 0.0D+00 )THEN
                        IERGAM = -3
                        GAMMAF = DFLAG
                     ELSE
                        IERGAM =  0
                        GAMMAF = Z / DENOM
                     ENDIF
               ENDIF
  
  
 99   RETURN
      END

      SUBROUTINE GETJK ( VALUEJ, VALUEK, IERGQ, IERROM )
  
C    Purpose -     Get the values of the emissivity and the absorption 
C       coefficient.  This routine sums the results of the integrations 
C       in GAMMA.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    IERGQ, IERROM
  
      REAL*8       CONJ, CONK, EMISIV, ABSORP, SUMEMI, SUMABS, 
     1             VALUEJ, VALUEK, DSREM, EMLAST, ABLAST

C                  Initialize values of sums of J and K to zero.
         SUMEMI = 0.00D+00
         SUMABS = 0.00D+00
C                  Initialize last calculated values of EMISIV and ABSORP
C                  to zero. 
	   EMLAST = 0.00D+00
	   ABLAST = 0.00D+00
C                  Initialize value of highest harmonic in sums to NSMAX.
         NSCON = NSMAX
  
         IF( CTHETA .EQ. 0.00D+00 )  THEN
C                  Get J and K for THETA = 90 degrees.
            DO  NS  = NSMIN, NSMAX
C                  Number of remaining harmonics not included in sums.
               DSREM = DFLOAT( NSMAX - NS + 1 )
C                  Get J and K for current NS.  Then test for errors.
               CALL GET90 ( EMISIV, ABSORP, IERGQ, IERROM )
                  IF( (IERGQ  .NE. 0) .OR. (IERPHI .NE. 0) .OR. 
     1                (IERXS  .NE. 0) .OR. (IERCAT .NE. 0) ) THEN
                     IEREMI = -NS
                     GOTO 99
                  ELSE

C	**************************************************************
C			Check EMISIV and ABSORP for each value of s.
C				   WRITE(10,*) NS, EMISIV, ABSORP
C	**************************************************************

C				Do not compute any more harmonics if convergence 
C					criterion is met.
					IF( (EMISIV .LT. EMLAST) .AND. 
     1					(ABSORP .LT. ABLAST) .AND.
     2				    (DSREM*EMISIV .LT. RERR*SUMEMI) .AND. 
     3					(DSREM*ABSORP .LT. RERR*SUMABS) ) THEN
						NSCON=NS
						SUMEMI = SUMEMI + EMISIV
						SUMABS = SUMABS + ABSORP
						GOTO 90
					ELSE
C                  Sum integrations of J and K for current NS.
                          EMLAST = EMISIV
	                    ABLAST = ABSORP
                          SUMEMI = SUMEMI + EMISIV
                          SUMABS = SUMABS + ABSORP
	                ENDIF
                  ENDIF
           END DO
        ELSE
C                  Get J and K for THETA not equal to 90 degrees.
            DO  NS  = NSMIN, NSMAX
C                  Number of remaining harmonics not included in sums.
               DSREM = DFLOAT( NSMAX - NS + 1 )
C                  Get J and K for current NS.  Then test for errors.
               CALL GETEMI ( EMISIV, ABSORP, IERGQ, IERROM )
                  IF( (IERGQ  .NE. 0) .OR. (IERCAT .NE. 0) ) THEN
                     IEREMI =  NS
                     GOTO 99
                  ELSE

C	**************************************************************
C			Check EMISV and ABSORP for each value of s.
C				   WRITE(10,*) NS, EMISIV, ABSORP
C	**************************************************************

C				Do not compute any more harmonics if convergence 
C					criterion is met.
					IF( (EMISIV .LT. EMLAST) .AND. 
     1					(ABSORP .LT. ABLAST) .AND.
     2				    (DSREM*EMISIV .LT. RERR*SUMEMI) .AND. 
     3					(DSREM*ABSORP .LT. RERR*SUMABS) ) THEN
						NSCON=NS
						SUMEMI = SUMEMI + EMISIV
						SUMABS = SUMABS + ABSORP
						GOTO 90
					ELSE
C                  Sum integrations of J and K for current NS.
                          EMLAST = EMISIV
	                    ABLAST = ABSORP
						SUMEMI = SUMEMI + EMISIV
						SUMABS = SUMABS + ABSORP
					ENDIF
                  ENDIF
           END DO
        ENDIF
  
C                  Get coefficients for J and K.
 90        CALL GETCON ( CONJ, CONK )
           VALUEJ = CONJ * SUMEMI
           VALUEK = CONK * SUMABS
  
  
 99   RETURN
      END

      SUBROUTINE GET90 ( EMISIV, ABSORP, IERGQ, IERROM )
  
C    Purpose -     Get the value of the integrals for the emissivity 
C       and the absorption coefficient, for THETA = 90 degrees, for the 
C       current value of S.  This routine makes the call to the 
C       subprogram function DMLIN and/or DCADRE.  
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV, IERGQ, IERROM
  
      REAL*8       ABSORP, EMISIV, GAMMAS, BETAS, ENDPTA, ENDPTB, 
     1             DMLIN, E90FCN, A90FCN, DCADRE, A90ROM, E90ROM
  
      PARAMETER    ( NINDV = 1 )
  
      DIMENSION    ENDPTA( NINDV ), ENDPTB( NINDV )
  
      COMMON       /EA90/ GAMMAS, BETAS
  
      EXTERNAL     E90FCN, A90FCN, E90ROM, A90ROM
  
  
C                  Set the limits or endpoints of the integrations.
            ENDPTA(1) = -1.00D+00
            ENDPTB(1) =  1.00D+00
  
C                  Get value of GAMMA and BETA for the current value of 
C                  the summing index, NS.
            GAMMAS = (DFLOAT( NS )) * (RNUB / RNU)
            BETAS  = DSQRT( (1.0D+00 - (1.0D+00/(GAMMAS**2))) )
  
  
C                  Use the Gaussian Quadrature method if METHOD = 1, or
C                  use the ROMBERG, ADAPTIVE method if METHOD = 2.
C				 For any other value of METHOD, first try Gaussian 
C				 Quadrature and, if that doesn't work, use the 
C				 Adaptive Romberg method.  
10          IF( METHOD .NE. 2 )  THEN
C                  Call the function integration routine DMLIN to 
C                  evaluate the integrals for the emissivity and the 
C                  absorption coefficient, resp.  Then test for error.
                  EMISIV = DMLIN ( E90FCN, ENDPTA, ENDPTB, NINDV, 
     1             MAXFCN, RERR, IERGQ )
                     IF( (METHOD .EQ. 1) .AND. (IERGQ .NE. 0) ) GOTO 99
                     IF( (METHOD .NE. 1) .AND. (IERGQ .NE. 0) ) THEN
					  METHOD = 2
					  GOTO 10
				   END IF
  
                  ABSORP = DMLIN ( A90FCN, ENDPTA, ENDPTB, NINDV, 
     1             MAXFCN, RERR, IERGQ )
                     IF( (METHOD .EQ. 1) .AND. (IERGQ .NE. 0) ) GOTO 99
                     IF( (METHOD .NE. 1) .AND. (IERGQ .NE. 0) ) THEN
					  IERGQ = -IERGQ
					  METHOD = 2
					  GOTO 10
				   END IF
            ELSE
C                  Call the function integration routine DCADRE to 
C                  evaluate the integrals for the emissivity and the 
C                  absorption coefficient, resp.  Then test for error.
                  EMISIV = DCADRE ( E90ROM, ENDPTA(1), ENDPTB(1), RERR, 
     1              ERROM, IERROM )
                     IF( IERROM .EQ. 100 )  GOTO 99
  
                  ABSORP = DCADRE ( A90ROM, ENDPTA(1), ENDPTB(1),RERR, 
     1              ERROM, IERROM )
                     IF( IERROM .EQ. 100 )  THEN
                        IERROM = - IERROM
                        GOTO 99
                     ENDIF
            ENDIF
  
C                  Multiply results from both integrations times the 
C                  square of BETA.
            EMISIV = (BETAS**2) * EMISIV
            ABSORP = (BETAS**2) * ABSORP
  
 99   RETURN
      END

      SUBROUTINE GETEMI ( EMISIV, ABSORP, IERGQ, IERROM )
  
C    Purpose -     Get the value of the integrals for the emissivity 
C       and the absorption coefficient, for the current value of S.
C       This routine makes the call to the subprogram function 
C       DMLIN and/or DCADRE.  
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV, IERGQ, IERROM
  
      REAL*8       ABSORP, EMISIV, DMLIN, DCADRE, EMIFCN, ABSFCN, 
     1             GAMMAI, GAMMAF, ENDPTA, ENDPTB, EMIROM, ABSROM
  
      PARAMETER    ( NINDV = 1 )
  
      DIMENSION    ENDPTA( NINDV ), ENDPTB( NINDV )
  
      EXTERNAL     EMIFCN, ABSFCN, EMIROM, ABSROM
  

C                  Get the limits of the integration in GAMMA for the 
C                  current value of NS (summing index).
            CALL GAMLIM ( GAMMAI, GAMMAF )
               IF( IERGAM .NE. 0 ) IERGQ = -1
  
C                  Set the limits or endpoints of the integrations.
            ENDPTA(1) = DMAX1 ( GAMMIN, GAMMAI )
            ENDPTB(1) = DMIN1 ( GAMMAX, GAMMAF )

		  IF( ENDPTA(1) .GE. ENDPTB(1) ) THEN

			EMISIV = 0.0D+00
			ABSORP = 0.0D+00

		  ELSE
  
C                  Use the Gaussian Quadrature method if METHOD = 1, or
C                  use the ROMBERG, ADAPTIVE method if METHOD = 2.
C				 For any other value of METHOD, first try Gaussian 
C				 Quadrature and, if that doesn't work, use the 
C				 Adaptive Romberg method.  
10            IF( METHOD .NE. 2 )  THEN
C                  Call the function integration routine DMLIN to 
C                  evaluate the integrals for the emissivity and the 
C                  absorption coefficient, resp.
                  EMISIV = DMLIN ( EMIFCN, ENDPTA, ENDPTB, NINDV, 
     1             MAXFCN, RERR, IERGQ )
                     IF(  (METHOD .NE. 1) .AND.  
     1				   ( (IERGQ .NE. 0) .OR. (IERPHI .NE. 0) 
     2                .OR. (IERXS .NE. 0) .OR. (IERCAT .NE. 0) 
     3                .OR. (IERBES .EQ. 140)  .OR. (IERBES .EQ. 141)
     4                .OR. (IERBES .EQ. 142) )  ) THEN
                        METHOD = 2
					  GOTO 10
                     ENDIF
                  			     
                  ABSORP = DMLIN ( ABSFCN, ENDPTA, ENDPTB, NINDV, 
     1             MAXFCN, RERR, IERGQ )
                     IF(  (METHOD .NE. 1) .AND.
     1			       ( (IERGQ .NE. 0) .OR. (IERPHI .NE. 0) 
     2                .OR. (IERXS .NE. 0) .OR. (IERCAT .NE. 0) 
     3                .OR. (IERBES .EQ. 140) .OR. (IERBES .EQ. 141 )
     4                .OR. (IERBES .EQ. 142) )  ) THEN
                        IERGQ = -IERGQ
	                  METHOD = 2
	                  GOTO 10
                     ENDIF
              ELSE
C                  Call the function integration routine DCADRE to 
C                  evaluate the integrals for the emissivity and the 
C                  absorption coefficient, resp.  Then test for errors.
C                  IF IERROM .EQ. 100 ABORT JOB BECAUSE ERROR IS SEVERE 
C			     (FATAL).
                  EMISIV = DCADRE ( EMIROM, ENDPTA(1), ENDPTB(1), RERR, 
     1              ERROM, IERROM )
                     IF(   (IERROM .NE.   0) .OR. (IERPHI .NE. 0) 
     1                .OR. (IERXS  .NE.   0) .OR. (IERCAT .NE. 0) 
     2                .OR. (IERBES .EQ. 140) .OR. (IERBES .EQ.141)
     3                .OR. (IERBES .EQ. 142) )  GOTO 99
  
                  ABSORP = DCADRE ( ABSROM, ENDPTA(1), ENDPTB(1),RERR, 
     1              ERROM, IERROM )
                     IF(   (IERROM .NE.   0) .OR. (IERPHI .NE. 0) 
     1                .OR. (IERXS  .NE.   0) .OR. (IERCAT .NE. 0) 
     2                .OR. (IERBES .EQ. 140) .OR. (IERBES .EQ.141)
     3                .OR. (IERBES .EQ. 142) )  THEN
                        IERROM = - IERROM
                        GOTO 99
                     ENDIF
              ENDIF
		  ENDIF

99    IF( (IERGQ .NE. 0) .OR. (IERROM .NE. 0) ) THEN
		WRITE(IUNITB,'(2I5)')IERGQ, IERROM
	ENDIF

      RETURN
      END

      FUNCTION E90FCN ( NINDV, COSPHI )
  
C    Purpose -   This routine governs the evaluation of the emissivity 
C       integrand.  This routine is accessed directly by the routine 
C	  DMLIN.  It calls to get the values of PHI-S, X-S, the 
C       bracketed term squared in the integrand, and the value of the 
C       distribution function itself.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV
  
      REAL*8       BETAS, GAMMAS, E90FCN, COSPHI, SINPHI, XS, YPHI, 
     1             FCN, FCNPHI, FCNGAM, BESSL, DBESSL
  
      COMMON       /EA90/ GAMMAS, BETAS
  
  
C                  Get SINE of PHI from COSPHI.  Then get XS.
            SINPHI = DSQRT( (1.0D+00 - (COSPHI**2)) )
            XS = (DFLOAT(NS)) * BETAS * RINDX * SINPHI
  
  
C                  Get the value of the Bessel function and its first 
C                  derivative, of order NS, at the position XS.
            CALL GETBES ( XS, BESSL, DBESSL )
               IF( (IERBES .EQ. 140) .OR. (IERBES .EQ. 141) .OR.
     1         (IERBES .EQ. 142) )  GOTO 99
  
  
C                  Get the appropriate value of Y-PHI, depending on the 
C                  wave polarization mode.
            IF( MODE .EQ. 1 )  THEN
               YPHI = (COSPHI**2) * ( BESSL**2)
            ELSE
               YPHI = (SINPHI**2) * (DBESSL**2)
            ENDIF
  
  
C                  Get the value of the distribution function and its 
C                  first derivatives for given values of GAMMA, BETA, 
C                  and COSPHI (cosine of phi).
            CALL DISTRN ( GAMMAS, BETAS, COSPHI, FCN, FCNPHI, FCNGAM )
  
C                  Calculate final value for emissivity at 90 degrees.
            E90FCN = YPHI * FCN
  
  
 99   RETURN
      END

      FUNCTION A90FCN ( NINDV, COSPHI )
  
C    Purpose -   This routine governs the evaluation of the absorption 
C       coefficient integrand.  This routine is accessed directly by  
C       the routine DMLIN.  It calls to get the values of PHI-S, X-S,
C	  the bracketed term squared in the integrand, and the value of
C	  the distribution function itself.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV
  
      REAL*8       BETAS, GAMMAS, A90FCN, COSPHI, SINPHI, XS, YPHI, 
     1             ATERM, BTERM, FCN, FCNPHI, FCNGAM, BESSL, DBESSL
  
      COMMON       /EA90/ GAMMAS, BETAS
  
  
C                  Get SINE of PHI from COSPHI.  Then get XS.
            SINPHI = DSQRT( (1.0D+00 - (COSPHI**2)) )
            XS = (DFLOAT(NS)) * BETAS * RINDX * SINPHI
  
  
C                  Check on the value of SINPHI.  If it is zero, then 
C                  the value of the absorption coefficient, A90FCN, is 
C                  zero.  This we know from examining the integrand's 
C                  behavior near SINPHI = 0, i.e., taking the limits.
            IF( SINPHI .EQ. 0.00D+00 )  THEN
               A90FCN = 0.00D+00
            ELSE
C                  Otherwise, evaluate the integrand, first get the 
C                  value of the Bessel function and its first 
C                  derivative, of order NS, at the position XS.
               CALL GETBES ( XS, BESSL, DBESSL )
                  IF( (IERBES .EQ. 140) .OR. (IERBES .EQ. 141) .OR.
     1            (IERBES .EQ. 142)) GOTO 99
  
  
C                  Get the appropriate value of Y-PHI, depending on the 
C                  wave polarization mode.
               IF( MODE .EQ. 1 )  THEN
                  YPHI = (COSPHI**2) * ( BESSL**2)
               ELSE
                  YPHI = (SINPHI**2) * (DBESSL**2)
               ENDIF
  
  
C                  Get the value of the distribution function and its 
C                  first derivatives for given values of GAMMA, BETA, 
C                  and COSPHI (cosine of phi).
               CALL DISTRN ( GAMMAS, BETAS, COSPHI, FCN, FCNPHI, 
     1          FCNGAM )
   
C                  Calculate final value for emissivity at 90 degrees.
               ATERM  =  - COSPHI / ( (BETAS**2) * GAMMAS * SINPHI )

	         BTERM = ( 1.0D+00 + ( 1.0D+00 / BETAS**2 ) ) / GAMMAS

	         A90FCN = (BTERM * FCN - FCNGAM) + (ATERM * FCNPHI)
               A90FCN = YPHI * A90FCN
            ENDIF
  
  
 99   RETURN
      END

      FUNCTION EMIFCN ( NINDV, GAMMA )
  
C    Purpose -   This routine governs the evaluation of the emissivity 
C       integrand.  This routine is accessed directly by the IMSL 
C       routine DMLIN.  It calls to get the values of PHI-S, X-S, the 
C       bracketed term squared in the integrand, and the value of the 
C       distribution function itself.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV
  
      REAL*8       BETA, GAMMA, EMIFCN, CPHIS, SPHIS, XS, VALUE, 
     1             FCN, FCNPHI, FCNGAM
  
  
C                  Get value of BETA from GAMMA.
            BETA  = DSQRT( (1.0D+00 - (1.0D+00/(GAMMA**2))) )
  
  
C                  Get value of COSINE & SINE of PHI-S.  Then check for 
C                  an error.
            CALL GETPHI ( GAMMA, BETA, CPHIS, SPHIS )
            IF( IERPHI .NE. 0 )  GOTO 99
  
  
C                  Get value of X-S.  Then check for an error.
            CALL GETXS ( BETA, CPHIS, SPHIS, XS )
            IF(  IERXS .NE. 0 )  GOTO 99
  
  
C                  Get value of the common term in the integrand, the 
C                  term that is squared.  Then check for an error.
            CALL INTGND ( BETA, CPHIS, SPHIS, VALUE, XS )
            IF( (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) )  GOTO 99
  
  
C                  Get the value of the distribution function and its 
C                  first derivatives for given values of GAMMA, BETA, 
C                  and COSPHI (cosine of phi).
            CALL DISTRN ( GAMMA, BETA, CPHIS, FCN, FCNPHI, FCNGAM )
  
  
C                  Calculate final value for emissivity at an arbitrary 
C                  angle, but not equal to 90 degrees.
            EMIFCN = VALUE * FCN
  
  
 99   RETURN
      END

      FUNCTION ABSFCN ( NINDV, GAMMA )
  
C    Purpose -    This routine governs the evaluation of the absorption 
C       coefficient integrand.  This routine is accessed directly by 
C       the routine DMLIN.  It calls to get the values of PHI-S, X-S,
C	  the bracketed term squared in the integrand, and the value of
C	  the distribution function and its derivatives by PHI and GAMMA.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      INTEGER*4    NINDV
  
      REAL*8       BETA, GAMMA, ABSFCN, CPHIS, SPHIS, XS, VALUE, 
     1             FCN, FCNPHI, FCNGAM, ATERM, BTERM
  
  
C                  Get value of BETA from GAMMA.
            BETA  = DSQRT( (1.0D+00 - (1.0D+00/(GAMMA**2))) )
  
  
C                  Get value of COSINE & SINE of PHI-S.  Then check for 
C                  an error.
            CALL GETPHI ( GAMMA, BETA, CPHIS, SPHIS )
               IF( IERPHI .NE. 0 )  GOTO 99
  
  
C                  Check on the value of SPHIS.  If it is zero, then 
C                  the value of the absorption coefficient, ABSFCN, is 
C                  zero.  This we know from examining the integrand's 
C                  behavior near SPHIS = 0, i.e., taking the limits.
            IF( SPHIS .EQ. 0.00D+00 )  THEN
               ABSFCN = 0.00D+00
            ELSE
C                  Otherwise, evaluate the integrand, first get the 
C                  value of X-S.  Then check for an error.
               CALL GETXS ( BETA, CPHIS, SPHIS, XS )
                  IF(  IERXS .NE. 0 )  GOTO 99
  
  
C                  Get value of the common term in the integrand, the 
C                  term that is squared.  Then check for an error.
               CALL INTGND ( BETA, CPHIS, SPHIS, VALUE, XS )
                  IF( (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) )  GOTO 99
  
  
C                  Get the value of the distribution function and its 
C                  first derivatives for given values of GAMMA, BETA, 
C                  and COSPHI (cosine of phi).
               CALL DISTRN ( GAMMA, BETA, CPHIS, FCN, FCNPHI, FCNGAM )
  
  
C                  Calculate final value for absorption coefficient at 
C                  an arbitrary angle, but not equal to 90 degrees.
               ATERM  = ((RINDX * BETA * CTHETA) - CPHIS)
               ATERM  = ATERM / ( (BETA**2) * GAMMA * SPHIS )

	         BTERM = ( 1.0D+00 + ( 1.0D+00 / BETA**2 ) ) / GAMMA

	         ABSFCN = (BTERM * FCN - FCNGAM) + (ATERM * FCNPHI)
               ABSFCN = VALUE * ABSFCN
            ENDIF
  
  
 99   RETURN
      END

      SUBROUTINE GETPHI ( GAMMA, BETA, CPHIS, SPHIS )
  
C    Purpose -     This routine gets the value of PHI-S, as a function 
C       of NS (or S), of GAMMA and the 3 input frequencies,  and of the 
C       wave normal angle.  
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       ATERM, BETA, BTERM, CPHIS, SPHIS, GAMMA
  
  
C                  Evaluate numerator, ATERM, and denominator, BTERM.
            ATERM = (DFLOAT(NS)) * (RNUB/RNU) / GAMMA
            ATERM = 1.00D+00 - ATERM
            BTERM = RINDX * BETA * CTHETA
  
  
C                  Test if denominator, BTERM, is zero.  If so, then 
C                  set error code.  Otherwise, evaluate CPHIS (cosine 
C                  of PHI-S).
               IF( BTERM .EQ. 0.00D+00 )  THEN
                  CPHIS  = DFLAG
                  SPHIS  = DFLAG
                  IERPHI = -1
               ELSE
                  CPHIS = ATERM / BTERM
*      if( (dabs(cphis)) .gt. 1.00d+00 )  then
*      write(13,222) ns, cphis
*      write(13,223) aterm, bterm
* 222  format( '    ns, cphis=', I5, 2x, 1PG25.18 )
* 223  format( ' aterm, bterm=', 1PG25.18, 2x,  1PG25.18 )
*      endif
C                  Test if magnitude of CPHIS is greater than 1. If so, 
C                  then test if it is due most likely to roundoff & 
C                  truncation error.  If DABS(CPHIS) is greater than 
C                  1.0001, then set error code, or else if DABS(CPHIS) 
C                  is between 1 and 1.0001, reset it to +1 or -1, 
C                  whichever is appropriate.  Otherwise, DABS(CPHIS) 
C                  is within the appropriate range, and no corrections 
C                  for roundoff & truncation errors need be applied.
                     IF( (DABS(CPHIS)) .GT. 1.0001D+00 )  THEN
                        CPHIS  = DFLAG
                        SPHIS  = DFLAG
                        IERPHI = -2
                     ELSE IF( (1.00D+00 .LT. (DABS(CPHIS))) 
     1                .AND.   ((DABS(CPHIS)) .LT. 1.0001D+00) )  THEN
C                  The magnitude of CPHIS is greater than 1, but not by 
C                  much and due most likely to roundoff error.
                           IF( CPHIS .LT. 0.00D+00 )  THEN
                              CPHIS = -1.00D+00
                           ELSE
                              CPHIS =  1.00D+00
                           ENDIF
C                  Set the value of sine(PHI-S), SPHIS.
                        SPHIS  = 0.00D+00
                     ELSE
C                  The magnitude of CPHIS is less than 1, so no problem 
C                  and we get the value of sine(PHI-S), SPHIS.
                        SPHIS  = DSQRT( (1.00D+00 - (CPHIS**2)) )
                     ENDIF
               ENDIF
  
  
      RETURN
      END

      SUBROUTINE GETXS ( BETA, CPHIS, SPHIS, XS )
  
C    Purpose -     This routine gets the value of X-S, as a function 
C       of NS (or S), of GAMMA and the 3 input frequencies, of the wave 
C       normal angle, and the value of PHI-S.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       ATERM, BETA, BTERM, CPHIS, SPHIS, XS
  
  
C                  Determine the value of the numerator.
            ATERM = RINDX * (DBLE(FLOAT(NS))) * BETA * STHETA * SPHIS
  
  
C                  Evaluate the position variable, XS, for either the 
C                  ORDINARY or EXTRAORDINARY mode.  First, determine 
C                  the value of the denominator for XS, called BTERM.  
C                  Then evaluate XS.  If BTERM is 0, then XS is 
C                  flagged, otherwise calculate XS.
            BTERM = 1.0D+00 - (RINDX * BETA * CTHETA * CPHIS)
               IF( (BTERM .EQ. 0.00D+00) )  THEN
                  XS    =  DFLAG
                  IERXS = -1
               ELSE
                  XS    = ATERM / BTERM
               ENDIF
  
  
C                  Computations completed for positions XS.  Return to 
C                  calling unit.
      RETURN
      END

      SUBROUTINE INTGND ( BETA, CPHIS, SPHIS, VALUE, XS )
  
C     Purpose -    To calculate the term common to both integrands of 
C                  the emissivity and absorption coefficient.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       BESSL, BETA, CPHIS, DBESSL, SPHIS, VALUE, XS
  
  
C                  Get the value of the Bessel function and its first 
C                  derivative, of order NS, at the position XS.
            CALL GETBES ( XS, BESSL, DBESSL )
  
  
C                  Test the values of variables to make sure they won't 
C                  cause an overflow.
            IF( (XS .EQ. 0.00D+00) .AND. (STHETA .EQ. 0.00D+00) 
     1       .AND. (NS .GT. 0) )  THEN 
C                  Under the conditions specified just above, determine 
C                  the value of the first term here based on whether 
C                  the index of refraction, RINDX, is zero or not.  If 
C                  RINDX is zero, then flag the output value.  
C                  Otherwise, if index of refraction is not zero, then 
C                  the value of (BESSL / STHETA), as STHETA goes to 
C                  zero, is 1.
               IF( RINDX .EQ. 0.00D+00 )  THEN
                  VALUE  = DFLAG
                  IERCAT = -1
               ELSE
C                  If (BESSL/STHETA) goes to one, in the limit as 
C                  XS goes to zero, then use this expression.
                  IF( NS .EQ. 1 ) THEN
                    VALUE  = ATHET * CTHETA - 1.00D+00
                    VALUE  = VALUE * (BETA * SPHIS / 2.00D+00)
                    VALUE  = (VALUE**2) / BETA
                    IERCAT = 0
	            ELSE
	              VALUE = 0.00D+00
	              IERCAT = 0
	            ENDIF
               ENDIF
            ELSE IF( (STHETA .EQ. 0.00D+00) )  THEN 
C                  Test if SIN-of-THETA is zero.  Set error code.
                  VALUE  = DFLAG
                  IERCAT = -2
            ELSE
C                  No problems with zeroes.  Calculate VALUE.
                  VALUE  = BESSL * ATHET / STHETA 
                  VALUE  = VALUE * ( (CTHETA/RINDX) - (BETA*CPHIS) )
                  VALUE  = VALUE - (BETA * SPHIS * DBESSL)
                  VALUE  = (VALUE**2) / BETA
                  IERCAT = 0
            ENDIF
  
  
C                  Return to calling unit.
      RETURN
      END

      SUBROUTINE GETBES ( XS, BESSL, DBESSL )
  
C    Purpose   -   To get the value of the Bessel function, of order 
C                  NS, and its first derivative, at the point XS.
C
C    Variables -   
C        BESSL     Value of Bessel function, of order NS, at position 
C                  XS.
C        DBESSL    Value of the first derivative of Bessel function, of 
C                  order NS, at position XS.
C        IERBES    Error return variable from routine MMBSJN.  
C                  MMBSJN is the name of the subroutine to calculate
C			     the values of Bessel functions.
C        IERXS     Error return variable from calculation of XS.  If 
C                  IERXS is non-zero, then no Bessel functions, or its 
C                  derivatives, are calculated.
C        NS        Order of Bessel function to calculate.
C        XS        Value of argument of Bessel functions.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       BESSL, DBESSL, XS, DNS
  
	DNS = DFLOAT(NS)

C                  GOAL:  Get value of Bessel function, and its
C				 derivative, of integer order NS, at the value XS.  
C                     Call routine MMBSJN to get the Bessel 
C                     function values.  
  
            CALL MMBSJN ( XS, DNS, BESSL, DBESSL, IERBES )
  
      RETURN
      END

      SUBROUTINE GETCON ( CONJ, CONK )
  
C    Purpose -     This routine gets the value of the coefficients 
C       before the sums of the integrals for the emissivity and the 
C       absorption coefficient.
  
      IMPLICIT NONE
  
      INCLUDE      'GETGSE.INC'
  
      REAL*8       CONA, CONB, CONJ, CONK
  
  
C                  Determine two constants common to the coefficients 
C                  for J (emissivity) and K (absorption coefficient).
            CONA = (4.0D+00 * (PI**2) * (ECHARG**2) * DENSTY) / CLIGHT
            CONB = CONA / EMASS
  
  
C                  Determine the coefficient for J (emissivity), CONJ, 
C                  and K (absorption coefficient), CONK.  The evaluation
C                  of these coefficients depends on the angle THETA.  
C                  The value of the coefficients for a THETA equal to 
C                  90 degrees (COSINE-THETA = 0) is determined 
C                  separately from other angles.
                IF( CTHETA .EQ. 0.00D+00 )  THEN
                    CONJ = CONA * RNUB * RINDX
                    CONK = CONB * RNUB / (RNU**2)
                ELSE
                    CONJ = CONA * RNU  / (DABS(CTHETA)) 
                    CONJ = CONJ / (1.0D+00 + (ATHET**2))
                    CONK = CONB / (RNU * RINDX * (DABS(CTHETA)))
                    CONK = CONK / (1.0D+00 + (ATHET**2))
                ENDIF
  
  
      RETURN
      END

      FUNCTION E90ROM ( COSPHI )
  
C     Purpose -    This function calculates the emissivity function's 
C        integrand, and it is called by the integration routine DCADRE.
C        DCADRE is the Romberg, adaptive method.  The value of cosine(THETA)
C	   is 0 for this function. This subprogram does one thing, basically:
C	   It interfaces with the subprogram function E90FCN, by incorporating
C	   the number of independent variables, NINDV, in the call to E90FCN.
C	   (This way, another subprogram to calculate the integrand does not 
C        need to be written.)
  
      IMPLICIT NONE
  
      INTEGER*4    NINDV
  
      REAL*8       E90ROM, E90FCN, COSPHI
  
      PARAMETER    ( NINDV = 1 )
  
  
C                  Call to E90FCN to evaluate E90ROM for the Romberg, 
C                  adaptive integration routine DCADRE.
            E90ROM = E90FCN ( NINDV, COSPHI )
  
  
      RETURN
      END

      FUNCTION A90ROM ( COSPHI )
  
C     Purpose -    This function calculates the absorption coefficient's 
C        integrand, and it is called by the integration routine DCADRE.
C        DCADRE is the Romberg, adaptive method.  The value of cosine(THETA)
C	   is 0 for this function.  This subprogram does one thing, basically:
C	   It interfaces with the subprogram function A90FCN, by incorporating
C	   the number of independent variables, NINDV, in the call to A90FCN.
C	   (This way, another subprogram to calculate the integrand does not 
C        need to be written.)
  
      IMPLICIT NONE
  
      INTEGER*4    NINDV
  
      REAL*8       A90FCN, A90ROM, COSPHI
  
      PARAMETER    ( NINDV = 1 )
  
  
C                  Call to A90FCN to evaluate A90ROM for the Romberg, 
C                  adaptive integration routine DCADRE.
            A90ROM = A90FCN ( NINDV, COSPHI )
  
  
      RETURN
      END

      FUNCTION ABSROM ( GAMMA )
  
C     Purpose -    This function calculates the absorption coefficient's 
C        integrand, and it is called by the integration routine DCADRE.
C	   DCADRE is the Romberg, adaptive method. This subprogram does one
C	   thing, basically:  It interfaces with the subprogram function
C	   ABSFCN, by incorporating the number of independent variables,
C	   NINDV, in the call to ABSFCN.  (This way, another subprogram
C	   to calculate the integrand does not need to be written.)
  
      IMPLICIT NONE
  
      INTEGER*4    NINDV
  
      REAL*8       ABSFCN, ABSROM, GAMMA
  
      PARAMETER    ( NINDV = 1 )
  
  
C                  Call to ABSFCN to evaluate ABSROM for the Romberg, 
C                  adaptive integration routine DCADRE.
            ABSROM = ABSFCN ( NINDV, GAMMA )
  
  
      RETURN
      END

      FUNCTION EMIROM ( GAMMA )
  
C     Purpose -    This function calculates the emissivity function's 
C        integrand, and it is called by the integration routine DCADRE.
C	   DCADRE is the Romberg, adaptive method.  This subprogram does
C	   one thing, basically:  It interfaces with the subprogram
C	   function EMIFCN, by incorporating the number of independent
C	   variables, NINDV, in the call to EMIFCN.  (This way, another
C	   subprogram to calculate the integrand does not need to be written.)
  
      IMPLICIT NONE
  
      INTEGER*4    NINDV
  
      REAL*8       EMIFCN, EMIROM, GAMMA
  
      PARAMETER    ( NINDV = 1 )
  
  
C                  Call to EMIFCN to evaluate EMIROM for the Romberg, 
C                  adaptive integration routine DCADRE.
            EMIROM = EMIFCN ( NINDV, GAMMA )
  
  
      RETURN
      END
