      PROGRAM GYROSPEC
  
C            VERSION 3             24 JULY 2000
C
C    Address questions and comments to
C         Gordon D. Holman
C         NASA/Goddard Space Flight Center
C         Code 682
C         Greenbelt, MD 20771
C         Office phone: (301) 286-4636
C		E-Mail: holman@stars.gsfc.nasa.gov
C
C    Purpose -
C         The program computes values of the emissivity and absorption 
C         coefficients, j and K, resp., from the gyrosynchrotron 
C         radiation formulas.  This program uses three subprograms,
C		based upon subprograms found in "Numerical recipies in
C		FORTRAN 77".  The first and second are DMLIN and DCADRE.  
C		These two evaluate the integrals in the expressions for j
C         and K.  The third routine is MMBSJN, which calculates the
C		values of Bessel functions at a given point.  DMLIN uses a
C	    Gaussian Quadrature method, and will use up to 256 Gauss
C	    points to evaluate the integral.  DCADRE employs the method
C         of Romberg extrapolation, and subdivides the interval of
C         integration until the error tolerance is satisfied. MMBSJN
C	    determines the Bessel functions via Steed's method. This 
C		program reports all errors, from any section of the code, to
C	    the user.  However, fatal errors cause the course of the
C	    calculations to abort immediately after the error has been 
C		reported to the user.
C                                                                        
C    Files used -  
C		DCADRE.FOR			Romberg integration routine
C		DMLIN.FOR			Gaussian Quadrature integration routine
C		FDBPL.FOR			Electron distribution function and its derivatives
C		GETGSE.FOR			Core Routines - Compute j & K
C		GETGSE.INC			Include file - common blocks, parameter definitions
C         GYROSPECTRUM.FOR	Main Program - this file
C		MMBSJN.FOR			Computes values of Bessel functions
C
C    Terminal I/O - 		                                             
C         IUNITA     Defines device unit number for reading information  
C                    interactively from the user terminal keyboard.      
C                    Currently assigned to unit number 5.                 
C         IUNITB     Defines device unit number for writing information  
C                    to user's terminal screen.  Currently assigned to   
C                    unit number 6.                                      
C                                                                        
C    Subroutines -                                                       
C         DISTRN     Electron distribution function and its first derivatives                          
C         GETGSE     Governs computation of gyrosynchrotron emission  
C
C    Documentation - 
C         GyroCode.rtf

      IMPLICIT NONE
  
      INCLUDE     'GETGSE.INC'
  
      INTEGER*4   IERGQ, IERROM, INU, TIME0, ITIME, ITICK, IERALL

	INTEGER*4   ITHETA
  
      REAL*8      VALUEJ, VALUEK, VALJEX, VALKEX, VALJOR, VALKOR, RNUX

	REAL*8		RADIEX, RADIOR, FLUXEX, FLUXOR, TBRIEX, TBRIOR, RADPOL

      REAL*8  	VALJEX1, VALJEX2, VALKEX1, VALKEX2, VALJOR1, VALJOR2

	REAL*8		VALKOR1, VALKOR2, GAMLOW, GAMHIGH, GAMBRK, BFIELD, D

	REAL*8		NTH

	COMMON		/DISTLIMS/GAMLOW, GAMHIGH, GAMBRK

C	*****************************************************************
	NAMELIST/input/BFIELD,NTH,RNUB,RNUP,DENSTY,P,Q,THETAD,
     1			   GAMLOW,GAMBRK,GAMHIGH,RERR,METHOD, D
C	*****************************************************************
  
	OPEN(unit=11,file="output.txt", status="REPLACE")
      OPEN(unit=12,file="error.txt", status="REPLACE")

      CALL SYSTEM_CLOCK(TIME0)

C			  ***Input parameters***

	GAMLOW  = 1.02D+00
      GAMBRK  = 1.20D+00
	GAMHIGH	= 11.0D+00

	BFIELD	= 500.D+00
	THETAD  = 45.00D+00
	DENSTY	= 2.0D+07
	NTH     = 2.0D+09
	P       = 7.0D+00
	Q	    = 5.0D+00
	D		= 2.0D+09

C			  ***Specify method of integration and relative error***

	METHOD  = 1
	RERR    = 0.001D+00

C				 Calculate the electron gyro and plasma frequencies.

	RNUB = 2.80D+06*BFIELD
	RNUP = 8.98D+03*DSQRT(NTH)
	
C                  Calculate extraordinary mode cutoff frequency.

      RNUX = DSQRT( (RNUP**2) + ((RNUB/2.0D+00)**2) )
      RNUX = RNUX + (RNUB/2.0D+00)

C				 Loop over observation angle to magnetic field (THETAD)

      DO ITHETA = 18, 18
	   
	   THETAD = DFLOAT(ITHETA) * 2.5D+00

C				 Loop over radio frequencies (RNU)

	DO INU = 0, 430

	   IF( (INU .GE. 0) .AND. (INU .LE. 60) ) THEN
	      RNU = DFLOAT(INU) * 0.1D+08 + 4.0D+08
         ELSE IF( (INU .GE. 61) .AND. (INU .LE. 360) ) THEN
	      RNU = DFLOAT(INU-60) * 0.01D+09 + 1.0D+09
         ELSE IF( (INU .GE. 361) .AND. (INU .LE. 420) ) THEN
	      RNU = DFLOAT(INU-360) * 0.1D+09 + 4.0D+09
         ELSE IF( (INU .GE. 421) .AND. (INU .LE. 430) ) THEN
	      RNU = DFLOAT(INU-420) * 0.1D+10 + 1.0D+10
	   ENDIF

C				 *** EXTRAORDINARY MODE CALCULATIONS ***

	   MODE = -1
	   IF( RNU .LE. RNUX ) THEN
	     VALJEX = 1.00D-30
	     VALKEX = 1.00D-30
	     RADIEX = 1.00D-30
	     FLUXEX = 1.00D-30
	     TBRIEX = 1.00D-30
	   ELSE
C                  Call to subroutine, GETGSE, to get the values for the
C                  gyrosynchrotron emission. This will be performed in
C                  two steps: once for GAMMA between GAMMIN and GAMBRK, 
C                  and again for GAMMA between GAMBRK and GAMMAX.  The
C                  results for these values of J and K will be added 
C                  together to obtain the final values of J and K in
C                  extraordinary mode.

           GAMMIN = GAMLOW
	     GAMMAX = GAMBRK
           CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
	     VALJEX1 = VALUEJ
	     VALKEX1 = VALUEK

	     GAMMIN = GAMBRK
           GAMMAX = GAMHIGH
	     CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
	     VALJEX2 = VALUEJ
	     VALKEX2 = VALUEK

	     VALJEX = VALJEX1 + VALJEX2
	     VALKEX = VALKEX1 + VALKEX2

           RADIEX = (VALJEX/VALKEX)*
     *    	 (1.00D+00 - DEXP(-VALKEX*D))
	     FLUXEX = RADIEX * 2.5D+00* D**2 / 2.238D+07
	     TBRIEX = 6.50931D+36 * RADIEX / RNU**2
C                  Check for any errors or warnings.
	     IF ( (IERATH .NE. 0) .OR. (IERIND .NE. 0) .OR. 
     1	       (IERNS .NE. 0) .OR. (IERGAM .NE. 0) .OR. 
     2		  (IEREMI .NE. 0) .OR. (IERGQ .NE. 0)  .OR.
     3          (IERPHI .NE. 0) .OR. (IERXS .NE. 0)  .OR. 
     4          (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) .OR.
     5          (IERBES .EQ. 141) .OR. (IERBES .EQ.142) .OR.  
     6		  (IERROM .NE. 0) .OR. (IERNU .NE. 0) ) THEN
	        IERALL = MODE
	     ELSE
	        IERALL = 0
	     ENDIF
	   ENDIF

C				 *** ORDINARY MODE CALCULATIONS ***

	   MODE = 1
         IF( RNU .LE. RNUP ) THEN
	     VALJOR = 1.00D-30
	     VALKOR = 1.00D-30
	     RADIOR = 1.00D-30
	     FLUXOR = 1.00D-30
	     TBRIOR = 1.00D-30
         ELSE
C                  Call to subroutine, GETGSE, to get the values for the
C                  gyrosynchrotron emission.  The above procedure for the
C                  extraordinory mode is repeated here for the ordinary mode.

           GAMMIN = GAMLOW
           GAMMAX = GAMBRK
           CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
	     VALJOR1 = VALUEJ
	     VALKOR1 = VALUEK

           GAMMIN = GAMBRK
	     GAMMAX = GAMHIGH
	     CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
		 VALJOR2 = VALUEJ
		 VALKOR2 = VALUEK

		 VALJOR = VALJOR1 + VALJOR2
		 VALKOR = VALKOR1 + VALKOR2
		 RADIOR = (VALJOR/VALKOR)*
     *    	 (1.00D+00 - DEXP(-VALKOR*D))
	     FLUXOR = RADIOR * 2.5D+00* D**2 / 2.238D+07
	     TBRIOR = 6.50931D+36 * RADIOR / RNU**2
C                  Check for any errors of warnings.  IERALL = 0 if no errors.
C                  If error in x-mode, IERALL = -1.   If error in o-mode, 
C                  IERALL = 1.  If error in both modes, IERALL = 2.
	     IF ( (IERATH .NE. 0) .OR. (IERIND .NE. 0) .OR. 
     1	       (IERNS .NE. 0) .OR. (IERGAM .NE. 0) .OR. 
     2		  (IEREMI .NE. 0) .OR. (IERGQ .NE. 0)  .OR.
     3          (IERPHI .NE. 0) .OR. (IERXS .NE. 0)  .OR. 
     4          (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) .OR. 
	5          (IERBES .EQ. 141) .OR. (IERBES .EQ. 142) .OR. 
     5		  (IERROM .NE. 0) .OR. (IERNU .NE. 0) ) THEN
	        IERALL = MODE - IERALL
	     ENDIF
	   ENDIF

C				 Calculate fractional polarization

	   RADPOL = (RADIOR - RADIEX) / (RADIOR + RADIEX)

C				 Write results to screen and files

	   WRITE(IUNITB,'(2ES9.2,4(3X,I10))') THETAD, RNU, NSMIN, NSMAX,  
     1                                      NSCON, IERALL
	   WRITE(11,'(13ES14.4)')  THETAD, RNU, VALJEX, VALJOR, VALKEX,  
     1   VALKOR, RADIEX, RADIOR, FLUXEX, FLUXOR, TBRIEX, TBRIOR, RADPOL

	   WRITE(12,'(ES14.4,12I5)') RNU, IERATH, IERIND, IERNS, IERGAM,
     1   IEREMI, IERGQ, IERPHI, IERXS, IERCAT, IERBES, IERROM, IERNU

	END DO
	END DO

      CALL SYSTEM_CLOCK(ITIME, ITICK)
      IF (ITICK /= 0)                                                  
     1WRITE (IUNITB,'(" Processing complete in ", F9.3, " seconds")')       
     2      REAL(ITIME-TIME0)/REAL(ITICK)

C	Write the NAMELIST parameters to the output file.
	WRITE(11,NML=input)
  
      STOP '0000'
      END
